{% load i18n pages_tags %}
<fieldset class="module aligned {{ fieldset.classes }} grp-module">
    {% if fieldset.name %}<h2 class="grp-collapse-handler">{{ fieldset.name }}</h2>{% endif %}
    {% if fieldset.description %}<div class="description">{{ fieldset.description|safe }}</div>{% endif %}

    {% for line in fieldset %}
        <div class="form-row{% if line.errors %} errors{% endif %}{% if not line.has_visible_field %} hidden{% endif %}{% for field in line %}{% if field.field.name %} field-{{ field.field.name }}{% endif %}{% endfor %}">
            {% for field in line %}
                <div class="field-box{% if field.field.name %} field-{{ field.field.name }}{% endif %}{% if not field.is_readonly and field.errors %} errors{% endif %}{% if field.field.is_hidden %} hidden{% endif %} {% if field.is_checkbox %} checkbox-row{% endif %}">
                    {{ field.errors }}
                    {% if field.is_checkbox %}
                        {{ field.field }}{{ field.label_tag }}
                    {% else %}
                        {{ field.label_tag }}
                        {% if field.is_readonly %}
                            <p>{{ field.contents }}</p>
                        {% else %}
                            {{ field.field }}
                        {% endif %}
                    {% endif %}
                    {% if field.field.help_text %}
                        <p class="help">{{ field.field.help_text|safe }}</p>
                    {% endif %}
                    {% for p in placeholders %}
                        {% ifequal p.ctype field.field.name %}
                            {% if not add %}
                                {% show_revisions page p.ctype language %}
                            {% endif %}
                        {% endifequal %}
                    {% endfor %}
                </div>
            {% endfor %}
        </div>
    {% endfor %}
</fieldset>
