{% extends "pages/tests/base.html" %}
{% load pages_tags %}
{% block parent %}
    {% block child %}
        {% placeholder one on current_page %}
        {% placeholder two on current_page %}
    {% endblock %}
{% endblock %}